# -*- coding: utf-8 -*-

# Copyright (C) 2015 Canonical Ltd.

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import json
from kombu import Connection
import os
import time


class TestClient(object):

    VALID_STATUSES = ('PASS',)

    EXCUSE_LABELS = {
        "PASS": '<span style="background:#87d96c">Pass</span>',
        "FAIL": '<span style="background:#ff6666">Regression</span>',
        "RUNNING": '<span style="background:#99ddff">Test in progress</span>',
    }

    def __init__(self, britney, distribution, series):
        self.britney = britney
        self.distribution = distribution
        self.series = series

    def _cache_file_name(self):
        """Returns the name of the cache file

        Provides the file name for write_cache and read_cache
        """
        home = os.path.expanduser("~")
        cache_file = os.path.abspath(os.path.join(home, 'testclient',
                                     '{}.json'.format(self.series)))
        return cache_file

    def announce_src(self, testing_excuses, amqp_uris):
        """Puts new src details in `pm.candidates` queue"""

        cached_entries = self._read_cache()

        to_be_announced = []
        for excuse in testing_excuses:
            if excuse in cached_entries:
                continue
            else:
                to_be_announced.append(excuse)

        with Connection(amqp_uris) as connection:
            try:
                queue = connection.SimpleQueue(
                    'pm.candidates')
                for candidate in to_be_announced:
                    payload = {
                        'source': candidate.name,
                        'version': candidate.version,
                        'distribution': self.distribution,
                        'series': self.series,
                    }
                    queue.put(payload)
                    self._write_cache({candidate.name, candidate.version})
            except Exception as exc:
                print("E: [%s] - Queueing package: %s erred with %s" %
                      (time.asctime(), source, exc))
            finally:
                queue.close()


    def _write_cache(self, candidate):
        """Writes new src details in to the cache file

        This helps decide if a package, version needs
        to be announced
        """
        cache_file = self._cache_file_name()

        if not os.path.exists(os.path.dirname(cache_file)):
            os.makedirs(os.path.dirname(cache_file))

        data = self._read_cache()
        to_remove = []

        with open(cache_file, 'w') as outfile:
            for cached_items in data:
                for key, value in cached_items.items():
                    # Remove the entries from the cache if
                    # another version of an entry is coming new
                    if candidate.name == key and candidate.version != value:
                        to_remove.append(cached_items)
            if {candidate.name: candidate.version} not in data:
                data.append({candidate.name: candidate.version})
            for removable in to_remove:
                data.remove(removable)
            json.dump(data, outfile)

    def _read_cache(self):
        """Reads the cache file contents

        This helps decide if a package, version needs
        to be announced
        """
        cache_file = self._cache_file_name()
        entries = []
        if not os.path.exists(cache_file):
            return entries
        with open(cache_file) as infile:
            data = json.load(infile)
            for k in data:
                entries.append(k)
        return entries
